<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="description" content="">
    <meta name="author" content="">
    <link rel="shortcut icon" href="{{ static_url('images/favicon.png') }}">

    <title>Butterfly</title>
    <link href="{{ static_url('stylesheets/main.css') }}" rel="stylesheet">
  </head>

  <body>
    <main>
    </main>

    <script src="{{ static_url('javascripts/term.js/src/term.js') }}"></script>
    <script src="{{ static_url('javascripts/main.js') }}"></script>
  </body>
</html>
